﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NDesk.Options;

namespace mailarch
{
    class Program
    {
        /* Parametry:
         *  -l = textovy soubor se seznamem souboru nebo adresaru
         *  -s = SMTP server
         *  -f = adresa odesilatele
         *  -t = adresa prijemce
         *  -a = jmeno archivu
         *  -p = heslo pro archivu
         */
        static void Main(string[] args)
        {
            // Zpracovani parametru.
            AppArguments app_arg = new AppArguments();
            bool help = false;

            var options = new OptionSet()
            {
                {"?|h|help", "Displays this help.", v => help = v != null},
                {"l|list=", "UTF-8 encoded file containing list of files or directories to zip. Each entry per line. Comments starts with `#' symbol.", v => app_arg.file_list = v},
                {"s|smtp=", "SMTP server.", v => app_arg.smtp_server = v},
                {"f|from=", "Sender's e-mail.", v => app_arg.sender = v},
                {"t|to=", "Recipient's e-mail.", v => app_arg.recipient = v},
                {"a|archive=", "Zip archive name.", v => app_arg.archive = v},
                {"p|password=", "Encrypt archive with password.", v => app_arg.password = v},
                {"ssl", "Use SSL when possible.", v => app_arg.ssl = v != null},
                {"dc", "Use default credentials.", v => app_arg.default_credentials = v != null}
            };

            try
            {
                options.Parse(args);
            }
            catch (OptionException e)
            {
                Console.WriteLine(e.Message);
                return;
            }

            // Zobrazeni napovedy a konec programu.
            if (help)
                ShowHelp(null, options);

            // Kontrola, jestli byly zadany vsechny parametry.
            if (String.IsNullOrEmpty(app_arg.file_list))
                ShowHelp("Missing file list.", options);

            if (String.IsNullOrEmpty(app_arg.smtp_server))
                ShowHelp("Specify SMTP server.", options);

            if (String.IsNullOrEmpty(app_arg.sender))
                ShowHelp("Specify sender's e-mail.", options);

            if (String.IsNullOrEmpty(app_arg.recipient))
                ShowHelp("Specify recipient's e-mail.", options);

            if (String.IsNullOrEmpty(app_arg.archive))
                ShowHelp("Missing ZIP archive name.", options);

            // Spusteni komprese a odeslani mailu.
            MailArch ma = new MailArch(app_arg);
            ma.Start();
        }

        static void ShowHelp(string message, OptionSet option_set)
        {
            if (!String.IsNullOrEmpty(message))
                Console.WriteLine(message + Environment.NewLine);

            Console.WriteLine("Usage: "
                + System.AppDomain.CurrentDomain.FriendlyName
                + " -l file -s SMTP_server -f sender_mail -t recipient_mail -a archive_name [-p archive_password]");

            Console.WriteLine("Arguments:");
            option_set.WriteOptionDescriptions(Console.Out);

            Console.WriteLine(Environment.NewLine
                + "This program contains following components:" + Environment.NewLine
                + "DotNetZip library: http://dotnetzip.codeplex.com/" + Environment.NewLine
                + "NDesk.Options parser: http://ndesk.org/Options");

            Environment.Exit(-1);
        }
    }
}
